﻿using System;
using System.Linq;
using System.Collections.Generic;
using Microsoft.AspNetCore.Mvc;

namespace PolicyFilesDeepDive.Controllers
{
    [Route("[controller]")]
    [ApiController]
    public class ApiController : ControllerBase
    {
        private readonly IValidValuesProvider _validValuesProvider = new StaticValidValuesProvider();

        [HttpGet("CompanyDomains")]
        public IActionResult GetCompanyDomains()
        {
            var domainsResponse = new
            {
                // The domain provider obtains a list of domains, perhaps from a database call, etc.
                Domains = _validValuesProvider.GetValidDomains()
            };
            var result = Ok(domainsResponse);
            return result;
        }

        public class ValidDomainsResponse
        {
            public IEnumerable<String> Domains { get; set; }
        }

        [HttpPost("StringCollectionContainsClaim")]
        public IActionResult CheckStringCollectionContainsClaim([FromBody] ContainsClaimRequest request)
        {
            var comparisonType = request.ignoreCase ? StringComparison.OrdinalIgnoreCase : StringComparison.Ordinal;
            var found = request.inputClaim.Any(x => x.Equals(request.item, comparisonType));
            var result = new ContainsClaimResponse { outputClaim = found };
            return Ok(result);
        }

        public class ContainsClaimRequest
        {
            public String[] inputClaim { get; set; } 
            public String item { get; set; }
            public Boolean ignoreCase { get; set; }
        }

        public class ContainsClaimResponse
        {
            public Boolean outputClaim { get; set; }
        }

        [HttpPost("VerifyAccessCode")]
        public IActionResult VerifyAccessCode([FromBody] VerifyAccessCodeRequest request)
        {
            // GetValidAccessCodeForUser an acces code given a user email adderss, perhaps from a database call, etc.
            var validAccessCodeForUser = _validValuesProvider.GetValidAccessCodeForUser(request.userEmail);

            if (String.Equals(request.accessCode, validAccessCodeForUser, StringComparison.OrdinalIgnoreCase))
            {
                return Ok();
            }

            // The codes did not match - return an error
            var errorContent = new ErrorResponse
            {
                version = "1.0",
                status = 409,
                code = "errorCode",
                requestId = "requestId",
                userMessage = "The provided code is invalid for the current user.",
                developerMessage = $"The provided code {request.accessCode} does not match the expected {validAccessCodeForUser} code for user {request.userEmail}.",
                moreInfo = "https://someurl.com/moreinformation"
            };
            var errorResult = new ConflictObjectResult(errorContent);
            return errorResult;
        }

        public class VerifyAccessCodeRequest
        {
            public String userEmail { get; set; }
            public String accessCode { get; set; }
        }

        public class ErrorResponse
        {
            public String version { get; set; }
            public Int32 status { get; set; }
            public String code { get; set; }
            public String userMessage { get; set; }
            public String developerMessage { get; set; }
            public String requestId { get; set; }
            public String moreInfo { get; set; }
        }
    }

    public interface IValidValuesProvider
    {
        IEnumerable<String> GetValidDomains();

        String GetValidAccessCodeForUser(String userEmail);
    }

    public class StaticValidValuesProvider : IValidValuesProvider
    {
        private static readonly String[] ValidDomains = new []
        {
            "yourdomain.com",
            "someotherdomain.com",
            "wintellect.com",
            "test.com"
        };

        public IEnumerable<String> GetValidDomains()
        {
            return ValidDomains;
        }

        public String GetValidAccessCodeForUser(String userEmail)
        {
            // Perhaps a little better than "12345", but not much...
            return "67890";
        }
    }
}